package es.imunnic.inversionitasBot;

import es.imunnic.inversionitasBot.services.ApiService;
import es.imunnic.inversionitasBot.services.GoogleNewsScraperService;
import es.imunnic.inversionitasBot.services.ResultadosService;
import es.imunnic.inversionitasBot.services.Trends24Scrapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.telegram.telegrambots.bots.TelegramLongPollingBot;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.exceptions.TelegramApiException;

import java.io.IOException;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class TelegramBot extends TelegramLongPollingBot {

  private static final Logger logger = LoggerFactory.getLogger(GoogleNewsScraperService.class);
  private GoogleNewsScraperService scrapper = new GoogleNewsScraperService();
  private Trends24Scrapper trendScrapper =  new Trends24Scrapper();
  private ResultadosService resultadosService = new ResultadosService();
  //@Value("${telegram.bot.username}")
  private String BOT_USERNAME = "Inversionitas_Bot";
  //@Value("${telegram.bot.token}")
  private String BOT_TOKEN = "7626026035:AAHEMp_iIN3y8AwywL0R6OTQvNi7EcJZ0iY";
  //private String BOT_TOKEN = "8017576757:AAHNGvY8RK_PbnlV1frM8KZMQ45tdMSvO3c";
  protected String CHAT_ID = "-1002289752202";
  protected Integer THREAD_INDICES = 138;
  protected Integer THREAD_NOTICIAS = 137;


  @Override
  public String getBotUsername() {
    return BOT_USERNAME;
  }

  @Override
  public String getBotToken() {
    return BOT_TOKEN;
  }

  @Override
  public void onUpdateReceived(Update update) {
    if (update.hasMessage() && update.getMessage().hasText()) {
      String messageText = update.getMessage().getText();
      String chatId = update.getMessage().getChatId().toString();

      if (messageText.equalsIgnoreCase("/resumen")) {
        construirResumenNoticias(chatId);
      } else if (messageText.toLowerCase().startsWith("/busca ")) {
        String query = messageText.substring(7).trim();
        buscarNoticiasYEnviar(chatId, query);
      } else if (messageText.toLowerCase().startsWith("/resultados ")) {
        String query = messageText.substring(12).trim();
        buscarResultadosYEnviar(chatId, query);
      } else if (messageText.toLowerCase().startsWith("/trends ")) {
        String country = messageText.substring(8).trim().toLowerCase().replace(" ", "-");
        enviarTrends(chatId, country);  // NUEVO
      } else {
        sendMessage(chatId, "Comando no reconocido. Usa /resumen para obtener el resumen diario.");
      }
    }
  }

  protected void sendMessage(String chatId, String text) {
    sendMessage(chatId, text, null); // Llama al otro método sin threadId
  }

  protected void sendMessage(String chatId, String text, Integer threadId) {
    List<String> partes = dividirMensaje(text);

    for (String parte : partes) {
      SendMessage sendMessage = new SendMessage();
      sendMessage.setChatId(chatId);
      sendMessage.setText(parte);
      sendMessage.setParseMode("MarkdownV2");

      // Agregar el ID del tema si es distinto de null
      if (threadId != null) {
        sendMessage.setMessageThreadId(threadId);
      }

      try {
        execute(sendMessage);
      } catch (TelegramApiException e) {
        e.printStackTrace();
      }
    }
  }

  protected void sendMessageHtml(String chatId, String text){
    this.sendMessageHtml(chatId, text, null);
  }

  protected void sendMessageHtml(String chatId, String text, Integer threadId) {
      String parte = this.escapeHtml(text);
      SendMessage sendMessage = new SendMessage();
      sendMessage.setChatId(chatId);
      sendMessage.setText(parte);
      sendMessage.setParseMode("HTML"); // Se debe usar HTML si el contenido es HTML


      if (threadId != null) {
        sendMessage.setMessageThreadId(threadId);
      }

      try {
        execute(sendMessage);
      } catch (TelegramApiException e) {
        e.printStackTrace();
      }
  }


  @Scheduled(cron = "0 0 13,20 * * ?", zone = "Europe/Paris")
  private void  mensajeProgramado(){
    this.construirResumenNoticias(this.CHAT_ID);
  }

  private void construirResumenNoticias(String chatid) {
    LocalDate today = LocalDate.now();
    String fechaInicio = today.minusDays(1).format(DateTimeFormatter.ISO_DATE);
    String fechaFin = today.plusDays(1).format(DateTimeFormatter.ISO_DATE);

    // Obtener las palabras clave de la API
    List<String> keywords = ApiService.getAllKeywords();

    // Mensaje base para el encabezado
    String encabezado = "📢 **Resumen de noticias relevantes de ayer y hoy:**\n\n";

    // Mensaje base para el pie
    String pie = "\n📢 Mañana volveremos con más información";

    // Enviar el encabezado primero
    sendMessage(chatid,encabezado,this.THREAD_NOTICIAS);

    // Enviar los mensajes para cada keyword
    for (int i = 0; i < keywords.size(); i++) {
      String keyword = keywords.get(i);
      StringBuilder mensaje = new StringBuilder();

      // Obtener las noticias para cada keyword (ahora se obtiene una lista de títulos)
      List<String> titles = ApiService.getTitlesByKeyword(keyword, fechaInicio, fechaFin);

      if (!titles.isEmpty()) {
        mensaje.append("🔹 **").append(keyword).append("**:\n");
        for (String title : titles) {
          title = title.replaceAll("([_*\\[\\]()~`>#+\\-=|{}.!])", "\\\\$1");

          mensaje.append("  🔹🔹 ").append(title).append("\n");
        }
      } else {
        mensaje.append("🔹 **").append(keyword).append("**: No hay noticias disponibles.\n");
      }

      // Enviar el mensaje para la palabra clave
      sendMessage(chatid, mensaje.toString(),this.THREAD_NOTICIAS);
    }

    // Enviar el pie al final
    sendMessage(chatid, pie,this.THREAD_NOTICIAS);
  }

  // Función para dividir el mensaje si excede el límite de caracteres de Telegram
  private List<String> dividirMensaje(String mensaje) {
    int maxLength = 4096;  // Límite de caracteres para un mensaje en Telegram
    List<String> partes = new ArrayList<>();

    // Divide el mensaje en partes de tamaño adecuado
    while (mensaje.length() > maxLength) {
      partes.add(mensaje.substring(0, maxLength));
      mensaje = mensaje.substring(maxLength);
    }

    // Agrega la última parte si queda algún mensaje restante
    if (!mensaje.isEmpty()) {
      partes.add(mensaje);
    }

    return partes;
  }

  private void buscarNoticiasYEnviar(String chatId, String query) {
    String mensajeInicio = "Analizando noticias... esto puede tardar un poco, ten paciencia...";
    String mensajeError = "Lo siento... se me ha licuado el cerebro... Mira los logs para saber más.";
    sendMessageHtml(chatId, mensajeInicio); // Enviar mensaje inicial

    try {
      List<NewsArticle> noticias = scrapper.searchNews(query);

      if (noticias.isEmpty()) {
        sendMessageHtml(chatId, "No encontré noticias sobre: " + query);
        return;
      }

      StringBuilder mensaje = new StringBuilder("<b>📰 Noticias sobre " + query + ":</b>\n\n");

      for (NewsArticle noticia : noticias) {
        mensaje.append("🔹 <a href=\"")
            .append(noticia.getUrl()) // URL de la noticia
            .append("\">")
            .append(escapeHtml(noticia.getTitle())) // Título de la noticia
            .append("</a>\n");
      }

      sendMessageHtml(chatId, mensaje.toString());
    } catch (Exception e) {
      sendMessageHtml(chatId, mensajeError);
    }
  }

  private void buscarResultadosYEnviar(String chatId, String query){
    Map<String, String> resultados = resultadosService.getBeneficioNetoAnual(query);

    if (resultados.isEmpty()) {
      sendMessageHtml(chatId, "<b>No se encontraron resultados financieros para:</b> " + query);
      return;
    }

    StringBuilder resultMessage = new StringBuilder();
    resultMessage.append("<b>Beneficio neto anual para: ").append(query).append("</b>\n\n");

    resultados.forEach((año, beneficio) -> {
      resultMessage.append("📅 <b>").append(año).append(":</b> ").append(beneficio).append("\n");
    });

    sendMessageHtml(chatId, resultMessage.toString());
  }

  // Método para escapar caracteres especiales en Markdown
  private String escapeMarkdown(String text) {
    if (text == null) {
      return "";
    }
    return text.replaceAll("([_*\\[\\]()~`>#+\\-=|{}.!])", "\\\\$1");
  }

  private String escapeHtml(String text) {
    return text.replace("&", "&amp;");
  }

  private void enviarTrends(String chatId, String country) {
    try {
      List<String> trends = trendScrapper.getTopTrends(country);

      if (trends.isEmpty()) {
        sendMessage(chatId, "No se encontraron tendencias para: " + country);
        return;
      }

      StringBuilder mensaje = new StringBuilder();
      mensaje.append("📈 <b>Top 20 tendencias en ").append(country.replace("-", " ")).append(":</b>\n\n");

      int index = 1;
      for (String trend : trends) {
        mensaje.append(index++).append(". ").append(escapeHtml(trend)).append("\n");
      }

      sendMessageHtml(chatId, mensaje.toString());
    } catch (IOException e) {
      sendMessage(chatId, "❌ Error al obtener las tendencias de Trends24, compruebe el nombre del pais");
      e.printStackTrace();
    }
  }

}
