package es.imunnic.inversionitasBot.services;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

@Service
public class ResultadosService {

  private static final Logger logger = LoggerFactory.getLogger(ResultadosService.class);
  private static final String USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36";

  /**
   * Realiza el scrap de los beneficios netos anuales de la empresa dada.
   *
   * @param companyName nombre de la empresa en la URL
   * @return Mapa con años como clave y beneficio neto como valor
   */
  public Map<String, String> getBeneficioNetoAnual(String companyName) {
    Map<String, String> resultados = new LinkedHashMap<>();
    String url = "https://es.investing.com/equities/" + companyName + "-financial-summary"; // Cambia esta URL a la real

    logger.info("Scrapeando resultados financieros de: {}", companyName);
    logger.info("URL objetivo: {}", url);

    try {
      Document doc = Jsoup.connect(url)
          .userAgent(USER_AGENT)
          .header("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8")
          .header("Accept-Language", "es-ES,es;q=0.9")
          .header("Referer", "https://www.google.com/")
          .proxy("51.44.176.151", 20202)
          .timeout(10000)
          .get();

      // Buscar la fila con el texto "Beneficio neto"
      Elements filas = doc.select("tr");
      for (Element fila : filas) {
        Element primeraColumna = fila.selectFirst("td");
        if (primeraColumna != null && primeraColumna.text().equalsIgnoreCase("Beneficio neto")) {
          Elements celdas = fila.select("td");
          for (int i = 2; i < celdas.size(); i++) { // Saltar los primeros <td> con el título
            String texto = celdas.get(i).text().trim();
            String año = String.valueOf(2020 + (i - 2)); // Ajusta esto si hay más o menos columnas
            if (!texto.equals("-") && !texto.isEmpty()) {
              resultados.put(año, texto);
              logger.info("Año {} → Beneficio neto: {}", año, texto);
            }
          }
          break;
        }
      }

    } catch (IOException e) {
      logger.error("Error durante el scraping de resultados de {}", companyName, e);
    }

    logger.info("Beneficios netos encontrados: {}", resultados.size());
    return resultados;
  }
}
