package es.imunnic.inversionitasBot.services;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class Trends24Scrapper {

    private static final String BASE_URL = "https://trends24.in/";

    /**
     * Obtiene los 20 primeros trending topics de Trends24 para el país especificado.
     * @param country El país en inglés, tal como aparece en la URL (ejemplo: "united-states").
     * @return Lista de trending topics (Strings).
     * @throws IOException Si falla la conexión o parsing.
     */
    public List<String> getTopTrends(String country) throws IOException {
      String url = BASE_URL + country + "/";
      Document doc = Jsoup.connect(url).get();

      List<String> trends = new ArrayList<>();

      Element trendList = doc.selectFirst("section#timeline ol.trend-card__list");
      if (trendList != null) {
        Elements trendElements = trendList.select("li .trend-name a");
        for (int i = 0; i < Math.min(20, trendElements.size()); i++) {
          Element trend = trendElements.get(i);
          trends.add(trend.text());
        }
      } else {
        System.out.println("❌ No se encontró la lista de tendencias.");
      }

      return trends;
    }
  }

